<?php

/**
 * ADOdb Lite is a PHP class to encapsulate multiple database APIs and is compatible with 
 * a subset of the ADODB Command Syntax. 
 * Currently supports Frontbase, MaxDB, miniSQL, MSSQL, MSSQL Pro, MySQLi, MySQLt, MySQL, PostgresSQL,
 * PostgresSQL64, PostgresSQL7, SqLite and Sybase.
 * 
 * @version 0.02
 */

$ADODB_vers = 'V0.04 ADOdb Lite 18 June 2005  (c) 2005 Mark Dickenson. All rights reserved. Released LGPL.';

define('ADODB_FETCH_DEFAULT',0);
define('ADODB_FETCH_NUM',1);
define('ADODB_FETCH_ASSOC',2);
define('ADODB_FETCH_BOTH',3);

GLOBAL $ADODB_FETCH_MODE;
$ADODB_FETCH_MODE = ADODB_FETCH_DEFAULT;	// DEFAULT, NUM, ASSOC or BOTH. Default follows native driver default...

/**
 * Database connection
 * Usage: $db = new ADONewConnection('dbtype');
 * 
 * @access public 
 * @param string $dbtype 
 */

function &ADONewConnection( $dbtype = 'mysql' )
{
	return new ADOConnection( $dbtype );
}

/**
 * Alternative Database connection
 * Usage: $db = new NewADOConnection('dbtype');
 * 
 * @access public 
 * @param string $dbtype 
 */

function &NewADOConnection($dbtype='')
{
	$tmp =& ADONewConnection($dbtype);
	return $tmp;
}

class ADOConnection
{
	var $connectionId;
	var $database;
	var $dbtype;
	var $host;
	var $open;
	var $password;
	var $username;
	var $persistent;
	var $record_set;
	var $debug = 0;
	var $forcenewconnection = false;
	var $extention;

	function ADOConnection( $dbtype )
	{
		$this->dbtype = strtolower( $dbtype );
	} 

	/**
	 * Returns floating point version number of ADOdb Lite
	 * Usage: $db->Version();
	 * 
	 * @access public 
	 */

	function Version()
	{
		global $ADODB_vers;
		return (float) substr($ADODB_vers,1);
	}

	/**
	 * Returns true if connected to database
	 * Usage: $db->IsConnected();
	 * 
	 * @access public 
	 */

	function IsConnected()
	{
    	return !empty($this->connectionId);
	}

	/**
	* Set how select queries will retrieve data.
	* Usage: $oldmode = $db->SetFetchMode($mode)
	*
	* @param mode	The fetchmode ADODB_FETCH_ASSOC or ADODB_FETCH_NUM
	* @returns		The previous fetch mode
	*/
	function SetFetchMode($mode)
	{	
		GLOBAL $ADODB_FETCH_MODE;
		$old = $ADODB_FETCH_MODE;
		$ADODB_FETCH_MODE = $mode;
		return $old;
	}

	/**
	 * Normal Database connection
	 * Usage: $result = $db->Connect('host', 'username', 'password', 'database');
	 * 
	 * @access public 
	 * @param string $database 
	 * @param string $host 
	 * @param string $password 
	 * @param string $username 
	 * @param string $forcenew // private 
	 */

	function Connect( $host = "", $username = "", $password = "", $database = "", $forcenew = false)
	{
		if ($host != "") $this->host = $host;
		if ($username != "") $this->username = $username;
		if ($password != "") $this->password = $password;
		if ($database != "") $this->database = $database;		
		$this->persistent = 0;
		$this->forcenewconnection = $forcenew;

		return $this->_connect();
	} 

	/**
	 * Persistent Database connection
	 * Usage: $result = $db->PConnect('host', 'username', 'password', 'database');
	 * 
	 * @access public 
	 * @param string $database 
	 * @param string $host 
	 * @param string $password 
	 * @param string $username 
	 */

	function PConnect( $host = "", $username = "", $password = "", $database = "")
	{
		if ($host != "") $this->host = $host;
		if ($username != "") $this->username = $username;
		if ($password != "") $this->password = $password;
		if ($database != "") $this->database = $database;		
		$this->persistent = 1;

		return $this->_connect();
	} 

	/**
	 * Force New Database connection
	 * Usage: $result = $db->NConnect('host', 'username', 'password', 'database');
	 * 
	 * @access public 
	 * @param string $database 
	 * @param string $host 
	 * @param string $password 
	 * @param string $username 
	 */

	function NConnect( $host = "", $username = "", $password = "", $database = "")
	{
		if ($host != "") $this->host = $host;
		if ($username != "") $this->username = $username;
		if ($password != "") $this->password = $password;
		if ($database != "") $this->database = $database;		
		$this->persistent = 0;
		$this->forcenewconnection = true;

		return $this->_connect();
	} 

	/**
	 * Open Database connection
	 * 
	 * @access private 
	 */

	function _connect()
	{
		include_once 'adodbSQL_drivers/' . $this->dbtype . '/' . $this->dbtype . '_driver.inc';
		if($this->dbOpen( $this->dbtype )) return true;

		$this->connectionId = false;

		return false;
	} 

	/**
	 * Connection to database server and selected database
	 * Set via $this->dbtype
	 * 
	 * @access private 
	 */

	function dbOpen( $dbtype )
	{
		include_once 'adodbSQL_drivers/' . $dbtype . '/dbOpen.inc';
		$extention = $dbtype . '_ADOConnection';
		$this->extention =& new $extention();
		$this->extention->parent = $this;
		return $result;		
	} 

	/**
	 * Choose a database to connect.
	 *
	 * @param dbname 	is the name of the database to select
	 * @return 		true or false
	 * @access public
	 */

	function SelectDB($dbname)
	{
		include_once 'adodbSQL_drivers/' . $dbname . '/SelectDB.inc';
		return $result;
	} 

	/**
	 * Returns SQL query and instantiates sql statement & resultset driver
	 * Driver set by $this->dbtype
	 * Usage: $linkId =& $db->execute( 'SELECT * FROM foo ORDER BY id' );
	 * 
	 * @access public 
	 * @param string $sql 
	 * @return mixed Resource ID, Array
	 */

	function &Execute( $sql, $inputarr = false )
	{
		$rs =& $this->SelectLimit($sql, -1, -1, $inputarr);
		return $rs;
	} 

	/**
	 * Returns SQL query and instantiates sql statement & resultset driver
	 * Driver set by $this->dbtype
	 * Usage: $linkId =& $db->SelectLimit( 'SELECT * FROM foo ORDER BY id', $nrows, $offset );
	 *        $nrows and $offset are optional
	 * 
	 * @access public 
	 * @param string $sql 
	 * @param string $nrows 
	 * @param string $offset 
	 * @return mixed Resource ID, Array
	 */

	function &SelectLimit( $sql, $nrows=-1, $offset=-1, $inputarr=false, $secs2cache=0 )
	{
		$stmt = $this->dbtype . 'Statement';
		$stmt = new $stmt( $sql, $this->connectionId, $offset, $nrows );
		$rs = $stmt->do_query(); 
		$this->record_set = $rs;
		return $rs;
	} 

	/**
	 * Returns the last record id of an inserted item
	 * Usage: $db->Insert_ID();
	 * 
	 * @access public 
	 */

	function Insert_ID()
	{
		include_once 'adodbSQL_drivers/' . $this->dbtype . '/Insert_ID.inc';
		return $insertid;
	}

	/**
	 * Closes database connection
	 * Usage: $db->close();
	 * 
	 * @access public 
	 */

	function close()
	{
		if ( $this->dbtype == 'postgres' || $this->dbtype == 'postgres64' || $this->dbtype == 'postgres7')
			$this->dbtype = 'pg';

		$closeConnection = $this->dbtype . '_close';
		$this->connectionId = @$closeConnection( $this->connectionId );
	}

	/**
	 * Return database error message
	 * Usage: $errormessage =& $db->ErrorMsg();
	 * 
	 * @access public
	 */

	function ErrorMsg()
	{
		return $this->extention->_errormsg();
	}

	/**
	 * Return database error number
	 * Usage: $errorbo =& $db->ErrorNo();
	 * 
	 * @access public
	 */

	function ErrorNo()
	{
		return $this->extention->_errorno();
	}

	/**
	 * Returns # of affected rows from insert/delete/update query
	 * 
	 * @access public 
	 * @return integer Affected rows
	 */

	function Affected_Rows()
	{
		return $this->extention->_affectedrows();
	} 

	 /**
	 * Returns All Records in an array
	 *
	 * Usage: $db->GetAll($sql);
	 * @access public 
	 */

	function &GetAll($sql, $inputarr = false)
	{
		$data =& $this->GetArray($sql, $inputarr);
		return $data;
	}

	 /**
	 * Returns All Records in an array
	 *
	 * Usage: $db->GetArray($sql);
	 * @access public 
	 */

	function &GetArray($sql, $inputarr = false)
	{
		$data = false;
		$result =& $this->Execute($sql, $inputarr);
		if ($result)
		{
			$data =& $result->GetArray();
			$result->Close();
		}
		return $data;
	}

	/**
	 * Returns the last record id of an inserted item
	 * Usage: $db->GetCol($sql);
	 * 
	 * @access public 
	 */

	function GetCol($sql, $inputarr = false, $trim = false)
	{
		$data = false;
		$result = &$this->Execute($sql, $inputarr);
		if ($result) {
			$data = array();
			while (!$result->EOF) {
				$data[] = ($trim) ? trim(reset($result->fields)) : reset($result->fields);
				$result->MoveNext();
			}
			$result->Close();
		}
		return $data;
	}

	 /**
	 * Return first element of first row of sql statement. Recordset is disposed
	 * for you.
	 *
	 * Usage: $db->GetOne($sql);
	 * @access public 
	 */

	function &GetOne($sql, $inputarr = false)
	{
		$data =& $this->GetRow($sql, $inputarr, true);
		return $data;
	}

	 /**
	 * Return one row of sql statement. Recordset is disposed for you.
	 *
	 * Usage: $db->GetRow($sql);
	 * @access public 
	 */

	function &GetRow($sql, $inputarr = false, $getone = false)
	{
		$result =& $this->Execute($sql, $inputarr);
		if ($result) {
			$result->Close();
			if ($getone)
			{
				if (!$result->EOF) return reset($result->fields);
			}
			else
			{
				if (!$result->EOF) return $result->fields;
				else return array();
			}
		}
		return false;
	}
}
?>