<?php

/**
 * ADOdb Lite is a PHP class to encapsulate multiple database APIs and is compatible with 
 * a subset of the ADODB Command Syntax. 
 * Currently supports Frontbase, MaxDB, miniSQL, MSSQL, MSSQL Pro, MySQLi, MySQLt, MySQL, PostgresSQL,
 * PostgresSQL64, PostgresSQL7, SqLite and Sybase.
 * 
 */

class msql_ADOConnection extends ADOConnection
{
	var $parent;

	function msql_ADOConnection()
	{
		// Dummy
	}

	/**
	 * Return database error message
	 * 
	 * @access private
	 */

	function _errormsg()
	{
		return @msql_error();
	}

	/**
	 * Return database error number
	 * 
	 * @access private
	 */
	function _errorno()
	{
		return (msql_error()) ? -1 : 0;
	}

	/**
	 * Returns # of affected rows from insert/delete/update query
	 * 
	 * @access private 
	 */

	function _affectedrows()
	{
		return @msql_affected_rows($this->parent->connectionId);
	} 
}

class msqlStatement
{
	var $connection;
	var $offset;
	var $nrows;
	var $sequence;
	var $sql;

	/**
	 * mysqlStatement Constructor
	 * 
	 * @access private 
	 * @param string $connection 
	 * @param string $parameters 
	 * @param string $sql 
	 */

	function msqlStatement( $sql, &$connection, $offset, $nrows )
	{
		$this->connection = &$connection;
		$this->offset = $offset;
		$this->nrows = $nrows;
		$this->sql = $sql;
	} 

	/**
	 * Executes SQL query and instantiates resultset methods
	 * 
	 * @access private 
	 * @return mixed Resultset methods
	 */

	function &do_query()
	{
		global $ADODB_FETCH_MODE;

//		$limit = '';
//		if ($this->offset != -1 || $this->nrows != -1)
//		{
//			$this->offset = ($this->offset>=0) ? $this->offset . "," : '';
//			$limit = ' LIMIT ' . $this->offset . ' ' . $this->nrows;
//		}

		$resultId = @msql_query( $this->sql );

		if ($resultId === false) { // error handling if query fails
			return false;
		} 

		if ($resultId === true) { // return simplified recordset for inserts/updates/deletes with lower overhead
			$rs =& new ADORecordSet_empty();
			return $rs;
		}

		$recordset = new msqlResultSet( $resultId, $this->connection );

		$recordset->_currentRow = 0;
		$recordset->sql = $this->sql;

		switch ($ADODB_FETCH_MODE)
		{
			case ADODB_FETCH_NUM: $recordset->fetchMode = MSQL_NUM; break;
			case ADODB_FETCH_ASSOC:$recordset->fetchMode = MSQL_ASSOC; break;
			default:
			case ADODB_FETCH_DEFAULT:
			case ADODB_FETCH_BOTH:$recordset->fetchMode = MSQL_BOTH; break;
		}

		$recordset->_fetch();

		return $recordset;
	} 
} 

/**
 * Empty result record set for updates, inserts, ect
 * 
 * @access private 
 */

class ADORecordSet_empty
{
	var $connectionId;
	var $fields;
	var $resultId;
	var $_currentRow = 0;
	var $_numOfRows = 0;
	var $_numOfFields = 0;
	var $fetchMode;
	var $EOF = true;
	function MoveNext() {return;}
	function RecordCount() {return 0;}
	function FieldCount() {return 0;}
	function EOF(){return TRUE;}
	function Close(){return true;}
	function fields() {return false;}
	function GetArray(){ 
		$results = array();
		return $results;
	}
	function Affected_Rows() { return 0;}
}

class msqlResultSet
{
	var $connectionId;
	var $fields;
	var $resultId;
	var $_currentRow = 0;
	var $_numOfRows = -1;
	var $_numOfFields = -1;
	var $fetchMode;
	var $EOF;
	var $sql;

	/**
	 * msqlResultSet Constructor
	 * 
	 * @access private 
	 * @param string $record 
	 * @param string $resultId 
	 */

	function msqlResultSet( $resultId, $connectionId )
	{
		$this->fields = array();
		$this->connectionId = $connectionId;
		$this->record = array();
		$this->resultId = $resultId;
		$this->EOF = false;
	} 

	/**
	 * Frees resultset
	 * 
	 * @access public 
	 */

	function close()
	{
		@msql_free_result( $this->resultId );
		$this->fields = array();
		$this->resultId = false;
	} 

	/**
	 * Returns field name from select query
	 * 
	 * @access public 
	 * @param string $field
	 * @return string Field name
	 */

	function fields( $field )
	{
		return $this->fields[$field];
	} 

	/**
	 * Returns numrows from select query
	 * 
	 * @access public 
	 * @return integer Numrows
	 */

	function RecordCount()
	{
		$this->_numOfRows = @msql_num_rows( $this->resultId );
		return $this->_numOfRows;
	} 

	/**
	 * Returns num of fields from select query
	 * 
	 * @access public 
	 * @return integer numfields
	 */

	function FieldCount()
	{
		$this->_numOfFields = @msql_num_fields( $this->resultId );
		return $this->_numOfFields;
	} 

	/**
	 * Returns next record
	 * 
	 * @access public 
	 */

	function MoveNext()
	{
		if (@$this->fields =& msql_fetch_array($this->resultId,$this->fetchMode)) {
			$this->_currentRow += 1;
			return true;
		}
		if (!$this->EOF) {
			$this->_currentRow += 1;
			$this->EOF = true;
		}
		return false;
	} 

	/**
	 * Move to the first row in the recordset. Many databases do NOT support this.
	 *
	 * @return true or false
	 */

	function MoveFirst() 
	{
		if ($this->_currentRow == 0) return true;
		return $this->Move(0);			
	}			

	/**
	 * Returns the Last Record
	 * 
	 * @access public 
	 */

	function MoveLast()
	{
		if ($this->EOF) return false;
		while (!$this->EOF) {
			$fields = $this->fields;
			$this->MoveNext();
		}
		$this->fields = $fields;
		return true;
	} 

	/**
	 * Random access to a specific row in the recordset. Some databases do not support
	 * access to previous rows in the databases (no scrolling backwards).
	 *
	 * @param rowNumber is the row to move to (0-based)
	 *
	 * @return true if there still rows available, or false if there are no more rows (EOF).
	 */

	function Move($rowNumber = 0) 
	{
		$this->EOF = false;
		if ($rowNumber == $this->_currentRow) return true;
		if ($rowNumber >= $this->_numOfRows)
	   		if ($this->_numOfRows != -1) $rowNumber = $this->_numOfRows-2;
  				
		if ($this->_seek($rowNumber)) {
			$this->_currentRow = $rowNumber;
			if ($this->_fetch()) {
				return true;
			}
		} else {
			$this->EOF = true;
			return false;
		}
	}

	/**
	 * Perform Seek to specific row
	 * 
	 * @access private 
	 */

	function _seek($row)
	{
		if ($this->_numOfRows == 0) return false;
		return @msql_data_seek($this->resultId,$row);
	}

	/**
	 * Fills field array with first database element when query initially executed
	 * 
	 * @access private 
	 */

	function _fetch()
	{
		$this->fields = @msql_fetch_array($this->resultId,$this->fetchMode);
		if( $this->RecordCount() == 0)
		{
			$this->EOF = true;
		}
		return is_array($this->fields);
	}

	/**
	 * Check to see if last record reached
	 * 
	 * @access public 
	 */

	function EOF()
	{
		if( $this->_currentRow < $this->_numOfRows)
		{
			return false;
		}
		else
		{
			$this->EOF = true;
			return true;
		}
	} 

	/**
	 * Returns All Records in an array
	 * 
	 * @access public 
	 * @param [nRows]  is the number of rows to return. -1 means every row.
	 */

	function GetArray($nRows = -1)
	{
		$results = array();
		$cnt = 0;
		while (!$this->EOF && $nRows != $cnt) {
			$results[] = $this->fields;
			$this->MoveNext();
			$cnt++;
		}
		return $results;
	} 

	function &GetRows($nRows = -1) 
	{
		$arr =& $this->GetArray($nRows);
		return $arr;
	}

	function &GetAll($nRows = -1)
	{
		$arr =& $this->GetArray($nRows);
		return $arr;
	}

} 
?>